CompositeMode

Enum CompositeMode 

pub enum CompositeMode {
    Replace,
    SrcOver,
}
The composition mode that should be used when rendering into a pixmap.

For performance reason it is highly recommended that you use CompositeMode::Replace, even if you know that the pixmap is already cleared. Only use SrcOver if you really have to preserve existing contents.

Variants



Replace

Clear the destination pixmap and render the scene into it.



SrcOver

Render the scene into the pixmap using src-over compositing.