pub struct RenderSettings {
pub level: Level,
pub num_threads: u16,
}Expand description
Settings to apply to the render context.
Fields
level: LevelThe SIMD level that should be used for rendering operations.
num_threads: u16The number of worker threads that should be used for rendering. Only has an effect
if the multithreading feature is active.